function X_beta=gfft(alph,varargin)
%GFFT Generalized Discrete Fourier transform.
%  GFFT(ALPH,X) is the generalized discrete Fourier transform (GDFT) of 
%  vector X, and parameter ALPH as described in [1]. For matrices, the 
%  GFFT operation is applied to each column. For N-D arrays, the GFFT 
%  operation operates on the first non-singleton dimension.
%
%  GFFT(ALPH,X,N) is the N-point GFFT, padded with zeros if X has less
%  than N points and truncated if it has more.
%
%  GFFT(ALPH,X,[],DIM) or GFFT(ALPH,X,N,DIM) applies the GFFT operation 
%  across the dimension DIM.
%
%  For length N input vector x, the GDFT is a length N vector X.   
%               N
%  X_beta(k) = sum  x(n)*exp(beta*(n-1))*exp(-j*2*pi*(k-1)*(n-1)/N), 
%              n=1                                            1 <= k <= N
%
%  where beta = log(alph)/N.
%
%  The inverse GDFT (computed by GIFFT) is given by
%                                 N
%  x(n) = [ exp(-beta*(n-1))/N ] sum  X_beta(k)*exp( j*2*pi*(k-1)*(n-1)/N),
%                                k=1                          1 <= n <= N.
%                                
%  GFFT makes use of the build-in function FFT. 
%
%
%  [1] J. Martinez, R. Heusdens and R. C. Hendriks, "A Generalized Poisson
%      Summation Formula and its Application to Fast Linear Convolution",
%      Signal Processing Letters, IEEE, vol 18, no. 9, sept 2011, 
%      pp. 501-504.
%
%
%  See also IGFFT, FFT, FFTSHIFT, FFTW, IFFT.

%  Copyright (c) 2011 Jorge Martinez, Msc. The Netherlands.
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2 of the License, or
%  (at your option) any later version.
%  
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%  
%  You should have received a copy of the GNU General Public License
%  along with this program; if not, write to the Free Software
%  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  
%  USA.

minargin=2;
maxargin=4;
minargout=0;
maxargout=1;

error(nargchk(minargin,maxargin,nargin,'struct'));      %Fast and basic test to see if we are getting called appropiatelly.
error(nargchk(minargout,maxargout,nargout,'struct'));   %Fast and basic test to see if we are getting called appropiatelly.

%%%%%%%%%%%% Checking mandatory arguments %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

CheckParameter(alph,'ALPH',true,true,nan); %Checking for non-empty, scalar, numeric parameter.

x=varargin{1};

CheckParameter(x,'X',false,false,nan); %Checking for numeric parameter.

%%%%%%%%%%%% Parameter initialization %%%%%%%%%%%%%%%%%%%%%%%

sx=size(x);


    
if nargin > 3 
    dim = varargin{3};
    CheckParameter(dim,'DIM',true,true,1); %Checking for non-empty, scalar, integer parameter greater than or equal to 1.
    if dim > ndims(x)
        sx=[sx ones(1,dim-ndims(x))];
    end
else

    dim = find(sx ~= 1,1);
    if isempty(dim)
        dim = 1;
    end
end

if nargin > 2 && ~isempty(varargin{2})
    N = varargin{2};
    CheckParameter(N,'N',false,true,0); %Checking for scalar, integer parameter greater than or equal to 0.   
    
    if sx(dim)<N
        padsize=zeros(length(sx),1);
        padsize(dim)=N-sx(dim);
        if any(sx==0)
            fsize=sx;
            fsize(dim)=N;
            X_beta=repmat([],fsize);
            return;
        end
        x=padarray(x,padsize,0,'post');
    end
    if sx(dim)>N
        ndimx=ndims(x);  
        idx = cell(1,ndimx);
        truncsize=sx;
        truncsize(dim)=N;
        if any(sx==0)
            X_beta=repmat([],truncsize);
            return;
        end

        for k= 1:ndimx;
            idx{k} = 1:truncsize(k);
        end
        x=x(idx{:});
    end
else
    N=sx(dim);
end


%%%%%%%%%%%% GFFT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n = (0:N-1).';
w = exp(log(alph).*n./N); % Exponential weight (window).
sx(dim)=1;
w=repmat(shiftdim(w,-(dim-1)),sx); % Modulation array.

X_beta=fft(x.*w,N,dim); %GFFT
end

function CheckParameter(par,Name,empt,scal,thres)
    if scal && (~isscalar(par) || ( empt && isempty(par)))
        me = MException(['AECToolbox:' mfilename ':scalar'], ...
                '%s must be a scalar',Name);
        throwAsCaller(me);    
    end

    if ~isnumeric(par)
        st = dbstack;
        name = regexp(st(2).name,'\.','split');
        me = MException(['MATLAB:' mfilename ':UndefinedFunction'], ...
                'Undefined function or method ''%s'' for input arguments of type ''%s''.',name{1},class(par));
        throwAsCaller(me);    
    end

    if  scal && ~isnan(thres) && (par < thres || logical(rem(par,1)))
        me = MException(['AECToolbox:' mfilename ':scalar'], ...
                '%s must be a scalar integer greater than or equal to %d',Name,thres);
        throwAsCaller(me);    
    end
    
    
end

