function x=igfft(alph,varargin)

%IGFFT Inverse Generalized Discrete Fourier transform.
%  IGFFT(ALPH,X_beta) is the inverse generalized discrete Fourier 
%  transform of vector X_beta and parameter ALPH as described in [1].
%
%  IGFFT(ALPH,X_beta,N) is the N-point inverse transform.
%
%  IGFFT(ALPH,X_beta,[],DIM) or IGFFT(ALPH,X_beta,N,DIM) is the inverse 
%  generalized discrete Fourier transform of X_beta across the 
%  dimension DIM.
%
%  IGFFT(..., 'symmetric') causes IGFFT to treat X_beta as conjugate 
%  symmetric along the active dimension.  This option is useful when 
%  X_beta is not exactly conjugate symmetric merely because of round-off 
%  error. This option must be used with care, since the introduction of 
%  parameter ALPH has influence on the conjugate symmetry of X_beta.
%  Refer to [1] and the reference page of the build-in IFFT function for 
%  more details on the specific mathematical definition of this symmetry.
%
%  IGFFT(..., 'nonsymmetric') causes IGFFT to make no assumptions about 
%  the symmetry of X_beta. This is the default behaviour.
%
%  IGFFT makes use of the build-in function IFFT. 
%
%
%  [1] J. Martinez, R. Heusdens and R. C. Hendriks, "A Generalized Poisson
%      Summation Formula and its Application to Fast Linear Convolution",
%      Signal Processing Letters, IEEE, vol 18, no. 9, sept 2011, 
%      pp. 501-504.
%
%  See also GFFT, IFFT, FFT, FFTSHIFT, FFTW.

%  Copyright (c) 2011 Jorge Martinez, Msc. The Netherlands.
%  This program is free software; you can redistribute it and/or modify
%  it under the terms of the GNU General Public License as published by
%  the Free Software Foundation; either version 2 of the License, or
%  (at your option) any later version.
%  
%  This program is distributed in the hope that it will be useful,
%  but WITHOUT ANY WARRANTY; without even the implied warranty of
%  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%  GNU General Public License for more details.
%  
%  You should have received a copy of the GNU General Public License
%  along with this program; if not, write to the Free Software
%  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  
%  USA.

minargin=2;
maxargin=5;
minargout=0;
maxargout=1;

error(nargchk(minargin,maxargin,nargin,'struct'));      %Fast and basic test to see if we are getting called appropiatelly.
error(nargchk(minargout,maxargout,nargout,'struct'));   %Fast and basic test to see if we are getting called appropiatelly.

%%%%%%%% Checking for a string argument in optional parameters %%%%%%%%%%

[reg, prop]=parseparams(varargin); %Checking for last 'symmetric' parameter.

if length(prop)>1
    error('AECToolbox:args', 'It can only be one string argument and must be the last one.')
end

sym_args{1}='symmetric';
sym_args{2}='nonsymmetric';

if ~isempty(prop) && ~any(strcmp(prop{1},sym_args))
    error('AECToolbox:char', 'Final string argument must be either "symmetric" or "nonsymmetric".')
end

if isempty(prop)
    sym_flag=sym_args{2};
else
    sym_flag=prop{1};
end

%%%%%%%%%%%% Checking mandatory arguments %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

CheckParameter(alph,'ALPH',true,true,nan); %Checking for non-empty, scalar, numeric parameter.

if ~isempty(reg)
    X_beta=reg{1};
else
    X_beta=prop{1};
end

CheckParameter(X_beta,'X_beta',false,false,nan); %Checking for numeric parameter.



%%%%%%%%%%%% Parameter initialization %%%%%%%%%%%%%%%%%%%%%%%

sx=size(X_beta);


    
if length(reg) > 2
    dim = varargin{3};
    CheckParameter(dim,'DIM',true,true,1); %Checking for non-empty, scalar, integer parameter greater than or equal to 1.
    if dim > ndims(X_beta)
        sx=[sx ones(1,dim-ndims(X_beta))];
    end
else

    dim = find(sx ~= 1,1);
    if isempty(dim)
        dim = 1;
    end
end

if length(reg) > 1 && ~isempty(varargin{2})
    N = varargin{2};
    CheckParameter(N,'N',false,true,0); %Checking for scalar, integer parameter greater than or equal to 0.   
    
    if sx(dim)<N
        padsize=zeros(length(sx),1);
        padsize(dim)=N-sx(dim);
        if any(sx==0)
            fsize=sx;
            fsize(dim)=N;
            x=repmat([],fsize);
            return;
        end
        X_beta=padarray(X_beta,padsize,0,'post');
    end
    if sx(dim)>N
        ndimx=ndims(X_beta);  
        idx = cell(1,ndimx);
        truncsize=sx;
        truncsize(dim)=N;
        if any(sx==0)
            x=repmat([],truncsize);
            return;
        end

        for k= 1:ndimx;
            idx{k} = 1:truncsize(k);
        end
        X_beta=X_beta(idx{:});
    end
else
    N=sx(dim);
end


%%%%%%%%%%%% IGFFT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

n = (0:N-1).';
w = exp(-log(alph).*n./N); % Inverse exponential weight (inverse window).
sx(dim)=1;
w=repmat(shiftdim(w,-(dim-1)),sx); % De-modulation array.

x=ifft(X_beta,N,dim,sym_flag).*w; %IGFFT
end

function CheckParameter(par,Name,empt,scal,thres)
    if scal && (~isscalar(par) || ( empt && isempty(par)))
        me = MException(['AECToolbox:' mfilename ':scalar'], ...
                '%s must be a scalar',Name);
        throwAsCaller(me);    
    end

    if ~isnumeric(par)
        st = dbstack;
        name = regexp(st(2).name,'\.','split');
        me = MException(['MATLAB:' mfilename ':UndefinedFunction'], ...
                'Undefined function or method ''%s'' for input arguments of type ''%s''.',name{1},class(par));
        throwAsCaller(me);    
    end

    if  scal && ~isnan(thres) && (par < thres || logical(rem(par,1)))
        me = MException(['AECToolbox:' mfilename ':scalar'], ...
                '%s must be a scalar integer greater than or equal to %d',Name,thres);
        throwAsCaller(me);    
    end
end

